class TestPass(object):
    '''Expose the ansible passwordstore plugin as pytest fixture'''
    def __init__(self, moleculebook):
        self._moleculebook = moleculebook

    def testpass(self, path):

        # reset playbook
        self._moleculebook.create(gather_facts=False,
                                  gather_roles=False,
                                  host='localhost')
        lookup = "{{ lookup('passwordstore', \'" + path + "\')}}"
        self._moleculebook.add_task_debug(lookup)
        result = self._moleculebook.run()[0]['msg']
        return result



from ansible import context
from ansible.module_utils.common.collections import ImmutableDict
from ansible.playbook.play import Play
from testaid.ansiblerun import AnsibleRun


class MoleculePlay(object):
    '''Run ansible playbooks against molecule host using the ansible python api.
    '''
    def __init__(self,
                 ansibleloader,
                 ansibleinventory,
                 ansiblevarsmanager,
                 ansiblehost,
                 moleculeenv):
        # Leverage the ansible python api
        # to run a playbook against a molecule host.
        #
        # see: ansible python api
        # https://docs.ansible.com/ansible/latest/dev_guide/developing_api.html

        self._moleculeenv = moleculeenv

        context.CLIARGS = ImmutableDict(connection='local',
                                        module_path=[''],
                                        forks=10,
                                        become=None,
                                        become_method=None,
                                        become_user=None,
                                        check=False,
                                        diff=False)
        self._loader = ansibleloader
        self._inventory = ansibleinventory
        self._variable_manager = ansiblevarsmanager
        self._host = ansiblehost

    def get_host(self):
        return self._host

    def get_molecule_scenario_directory(self):
        return self._moleculeenv.get_molecule_scenario_directory()

    def get_roles(self):
        return self._moleculeenv.get_roles()

    def read_vars(self, playbook):
        '''Return ansible variables without running a playbook.'''
        play = self._get_play_(playbook)
        ansible_vars = self._variable_manager.get_vars(play=play,
                                                       host=self._host)
        return ansible_vars

    def run_playbook(self, playbook):
        '''Run an ansible playbook using the ansible python api.'''
        play = self._get_play_(playbook)
        ansiblerun = AnsibleRun(self._inventory,
                                self._variable_manager,
                                self._loader)
        result = ansiblerun.run(play)
        return result

    def _get_play_(self, playbook):
        return Play().load(playbook,
                           variable_manager=self._variable_manager,
                           loader=self._loader)



import json


class TestVars(object):
    '''Expose ansible variabless of a molecule scenario.

    Include ansible facts form molecule host.
    Include roles from ansible project directory.
    Resolve jinja2 template variables.
    '''

    def __init__(self,
                 moleculelog,
                 debug_jsonvars,
                 moleculebook,
                 jsonvars,
                 gatherfrom_moleculehost,
                 gather_facts,
                 extra_vars):

        # this variable will be returned by the testvars fixture
        self._testvars = dict()

        if gatherfrom_moleculehost:
            run_playbook = True
        else:
            run_playbook = False

        # get ansible variables
        testvars_unresolved = moleculebook.get_vars(run_playbook,
                                                    gather_facts,
                                                    extra_vars)

        # convert python variables to json
        testvars_unresolved_json = json.dumps(testvars_unresolved)

        # set jsonvars to unresolved testvars
        jsonvars.set(testvars_unresolved_json)

        # resolve unresolved json testvars
        jsonvars.resolve()

        # print jsonvars debug info if command line flag is specified
        if debug_jsonvars:
            moleculelog.debug(jsonvars.get_debug())

        # get resolved testvars from jsonvars
        testvars_resolved_json = jsonvars.get()

        # convert json vars to python
        self._testvars = json.loads(testvars_resolved_json)

    def get_testvars(self):
        return self._testvars

    def get_cache(request, cache_key):
        try:
            # read testvars from cache
            # you can enable cache support in molecule.yml:
            # molecule -> verifier -> options
            # option "p: cacheprovider"
            testvars = request.config.cache.get(cache_key, None)
        except AttributeError:
            testvars = None
        return testvars

    def set_cache(self, request, cache_key):
        try:
            request.config.cache.set(cache_key, self._testvars)
        except AttributeError:
            pass



from ansible.inventory.host import Host
from ansible.inventory.manager import InventoryManager
from ansible.parsing.dataloader import DataLoader
from ansible.vars.manager import VariableManager


class AnsibleLoader(object):

    def __init__(self):
        pass

    def get(self):
        return DataLoader()


class AnsibleInventory(object):

    def __init__(self,
                 loader,
                 inventoryfile):
        self._inventory = InventoryManager(loader=loader,
                                           sources=str(inventoryfile))

    def get(self):
        return self._inventory


class AnsibleVarsManager(object):

    def __init__(self,
                 loader,
                 inventory):
        self._variable_manager = VariableManager(loader=loader,
                                                 inventory=inventory)

    def get(self):
        return self._variable_manager


class AnsibleLocalHost(object):

    def __init__(self):
        host = 'localhost'
        self._host = Host(name=host)

    def get(self):
        return self._host


class AnsibleHost(object):

    def __init__(self, inventory):
        host = next(iter(inventory.hosts))
        self._host = Host(name=host)

    def get(self):
        return self._host



import testaid.ansibleres
import testaid.ansiblerun
from testaid.boilerplate import hosts  # noqa F401
import testaid.exceptions
import testaid.jsonvars
import testaid.moleculebook
import testaid.moleculeenv
import testaid.moleculelog
import testaid.moleculeplay
import testaid.plugin
import testaid.templates
import testaid.testpass
import testaid.testvars  # noqa F401

name = "testaid"



class AnsibleRunError(Exception):
    '''Exception raised by MoleculePlay signals failed ansible playbook run.'''

    def __init__(self, msg, error):
        Exception.__init__(self, msg + '\n' + error)


class MoleculeBookRunError(Exception):
    '''Exception raised by MoleculeBook signals failed ansible playbook run.'''

    def __init__(self, msg):
        Exception.__init__(self, msg)


class TemplatesResolveError(Exception):
    '''Exception raised by TestVars signals failed ansible playbook run.'''

    def __init__(self, msg):
        Exception.__init__(self, msg)
